/**
 * Copyright (c) 2010-2015, Grill Balazs, Istvan Rath and Daniel Varro
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * Contributors:
 * Grill Balazs - initial API and implementation
 * Peter Lunk - EMFScope support added
 */
package org.eclipse.viatra.query.testing.core;

import com.google.common.base.Objects;
import com.google.common.collect.Iterables;
import com.google.common.collect.Sets;
import java.util.Set;
import org.eclipse.emf.common.notify.Notifier;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.viatra.query.runtime.api.IPatternMatch;
import org.eclipse.viatra.query.runtime.api.IQuerySpecification;
import org.eclipse.viatra.query.runtime.api.ViatraQueryMatcher;
import org.eclipse.viatra.query.runtime.emf.EMFScope;
import org.eclipse.viatra.query.runtime.exception.ViatraQueryException;
import org.eclipse.viatra.query.testing.core.IMatchSetModelProvider;
import org.eclipse.viatra.query.testing.snapshot.MatchSetRecord;
import org.eclipse.viatra.query.testing.snapshot.QuerySnapshot;
import org.eclipse.xtext.xbase.lib.Functions.Function1;
import org.eclipse.xtext.xbase.lib.IterableExtensions;

/**
 * <b>Note</b> If no query scope is set up explicitly, the resource set of the snapshot model will be used as a test model scope.
 * 
 * @deprecated this class is deprecated, use {@link InitializedSnapshotMatchSetModelProvider} instead.
 */
@Deprecated
@SuppressWarnings("all")
public class SnapshotMatchSetModelProvider implements IMatchSetModelProvider {
  private final URI snapshotModel;
  
  public SnapshotMatchSetModelProvider(final URI snapshotModel) {
    this.snapshotModel = snapshotModel;
  }
  
  @Override
  public <Match extends IPatternMatch> MatchSetRecord getMatchSetRecord(final EMFScope scope, final IQuerySpecification<? extends ViatraQueryMatcher<Match>> querySpecification, final Match filter) throws ViatraQueryException {
    final String FQN = querySpecification.getFullyQualifiedName();
    final Set<QuerySnapshot> snapshot = Sets.<QuerySnapshot>newHashSet();
    Set<? extends Notifier> _scopeRoots = scope.getScopeRoots();
    for (final Notifier n : _scopeRoots) {
      boolean _matched = false;
      if (!_matched) {
        if (n instanceof ResourceSet) {
          _matched=true;
          Resource _resource = ((ResourceSet)n).getResource(this.snapshotModel, true);
          EList<EObject> _contents = _resource.getContents();
          Iterable<QuerySnapshot> _filter = Iterables.<QuerySnapshot>filter(_contents, QuerySnapshot.class);
          Iterables.<QuerySnapshot>addAll(snapshot, _filter);
        }
      }
      if (!_matched) {
        if (n instanceof Resource) {
          _matched=true;
          URI _uRI = ((Resource)n).getURI();
          boolean _equals = _uRI.equals(this.snapshotModel);
          if (_equals) {
            EList<EObject> _contents = ((Resource)n).getContents();
            Iterable<QuerySnapshot> _filter = Iterables.<QuerySnapshot>filter(_contents, QuerySnapshot.class);
            Iterables.<QuerySnapshot>addAll(snapshot, _filter);
          }
        }
      }
    }
    return this.<IPatternMatch>getMatchSetRecordsFromSnapshot(snapshot, FQN);
  }
  
  @Override
  public boolean updatedByModify() {
    return false;
  }
  
  @Override
  public <Match extends IPatternMatch> MatchSetRecord getMatchSetRecord(final ResourceSet resourceSet, final IQuerySpecification<? extends ViatraQueryMatcher<Match>> querySpecification, final Match filter) throws ViatraQueryException {
    final String FQN = querySpecification.getFullyQualifiedName();
    Resource _resource = resourceSet.getResource(this.snapshotModel, true);
    EList<EObject> _contents = _resource.getContents();
    final Iterable<QuerySnapshot> snapshot = Iterables.<QuerySnapshot>filter(_contents, QuerySnapshot.class);
    return this.<IPatternMatch>getMatchSetRecordsFromSnapshot(snapshot, FQN);
  }
  
  @Override
  public void dispose() {
  }
  
  private <Match extends IPatternMatch> MatchSetRecord getMatchSetRecordsFromSnapshot(final Iterable<QuerySnapshot> snapshot, final String FQN) {
    MatchSetRecord _xblockexpression = null;
    {
      boolean _isEmpty = IterableExtensions.isEmpty(snapshot);
      if (_isEmpty) {
        String _plus = (this.snapshotModel + " is not a Snapshot model");
        throw new IllegalArgumentException(_plus);
      }
      QuerySnapshot _head = IterableExtensions.<QuerySnapshot>head(snapshot);
      EList<MatchSetRecord> _matchSetRecords = null;
      if (_head!=null) {
        _matchSetRecords=_head.getMatchSetRecords();
      }
      final Function1<MatchSetRecord, Boolean> _function = new Function1<MatchSetRecord, Boolean>() {
        @Override
        public Boolean apply(final MatchSetRecord it) {
          String _patternQualifiedName = it.getPatternQualifiedName();
          return Boolean.valueOf(Objects.equal(FQN, _patternQualifiedName));
        }
      };
      final MatchSetRecord record = IterableExtensions.<MatchSetRecord>findFirst(_matchSetRecords, _function);
      if ((record == null)) {
        throw new IllegalArgumentException(((("Could not find snapshot for " + FQN) + " in ") + this.snapshotModel));
      }
      _xblockexpression = record;
    }
    return _xblockexpression;
  }
}
