/**
 * Copyright (c) 2010-2016, Abel Hegedus, IncQuery Labs Ltd.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *   Abel Hegedus - initial API and implementation
 */
package org.eclipse.viatra.query.tooling.ui.queryresult.handlers;

import com.google.inject.Inject;
import com.google.inject.Injector;
import java.util.HashSet;
import java.util.Set;
import org.eclipse.core.commands.AbstractHandler;
import org.eclipse.core.commands.ExecutionEvent;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IResource;
import org.eclipse.emf.common.util.TreeIterator;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IViewPart;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.IWorkbenchPartSite;
import org.eclipse.ui.IWorkbenchSite;
import org.eclipse.ui.handlers.HandlerUtil;
import org.eclipse.viatra.query.patternlanguage.helper.CorePatternLanguageHelper;
import org.eclipse.viatra.query.patternlanguage.patternLanguage.Pattern;
import org.eclipse.viatra.query.tooling.ui.queryregistry.index.XtextIndexBasedRegistryUpdater;
import org.eclipse.viatra.query.tooling.ui.queryresult.QueryResultView;
import org.eclipse.xtext.resource.XtextResource;
import org.eclipse.xtext.ui.editor.XtextEditor;
import org.eclipse.xtext.ui.editor.model.IXtextDocument;
import org.eclipse.xtext.util.concurrent.IUnitOfWork;
import org.eclipse.xtext.xbase.lib.CollectionLiterals;
import org.eclipse.xtext.xbase.lib.IteratorExtensions;
import org.eclipse.xtext.xbase.lib.Procedures.Procedure1;

/**
 * @author Abel Hegedus
 */
@SuppressWarnings("all")
public class LoadVqlPatternHandler extends AbstractHandler {
  @Inject
  Injector injector;
  
  @Override
  public Object execute(final ExecutionEvent event) throws ExecutionException {
    final IEditorPart editorPart = HandlerUtil.getActiveEditorChecked(event);
    IWorkbenchSite _activeSite = HandlerUtil.getActiveSite(event);
    IWorkbenchPage _page = _activeSite.getPage();
    final IViewPart resultView = _page.findView(QueryResultView.ID);
    if ((resultView instanceof QueryResultView)) {
      final QueryResultView queryResultView = ((QueryResultView) resultView);
      final boolean active = queryResultView.hasActiveEngine();
      boolean _and = false;
      if (!active) {
        _and = false;
      } else {
        _and = (editorPart instanceof XtextEditor);
      }
      if (_and) {
        final XtextEditor xtextEditor = ((XtextEditor) editorPart);
        final IResource resource = xtextEditor.getResource();
        IProject _project = resource.getProject();
        String _name = _project.getName();
        final String sourceId = (XtextIndexBasedRegistryUpdater.DYNAMIC_CONNECTOR_ID_PREFIX + _name);
        IXtextDocument _document = xtextEditor.getDocument();
        final IUnitOfWork<HashSet<String>, XtextResource> _function = new IUnitOfWork<HashSet<String>, XtextResource>() {
          @Override
          public HashSet<String> exec(final XtextResource state) throws Exception {
            final HashSet<String> fqns = CollectionLiterals.<String>newHashSet();
            final TreeIterator<EObject> iterator = state.getAllContents();
            final Procedure1<EObject> _function = new Procedure1<EObject>() {
              @Override
              public void apply(final EObject pattern) {
                if ((pattern instanceof Pattern)) {
                  String _fullyQualifiedName = CorePatternLanguageHelper.getFullyQualifiedName(((Pattern)pattern));
                  fqns.add(_fullyQualifiedName);
                  iterator.prune();
                }
              }
            };
            IteratorExtensions.<EObject>forEach(iterator, _function);
            return fqns;
          }
        };
        final Set<String> patternFQNs = _document.<HashSet<String>>readOnly(_function);
        queryResultView.loadQueriesIntoActiveEngineInBackground(patternFQNs, sourceId);
      } else {
        IWorkbenchPartSite _site = queryResultView.getSite();
        Shell _shell = _site.getShell();
        MessageDialog.openError(_shell, "Query loading failed", 
          "Please load a model into the Query Results view before loading queries!");
      }
    }
    return null;
  }
}
